@@ -6,8 +6,11 @@ from yota.validators import Check
 import json
 import copy
 
-
 class TrackingMeta(type):
+    reserved_attr_names = ('context', 'hidden', 'g_context', 'start_template',
+                           'close_template', 'auto_start_close', '_renderer',
+                           '_processor', 'name')
+
     def __init__(mcs, name, bases, dict):
         """ Process all of the attributes in the `Form` (or subclass)
         declaration and place them accordingly. This builds the internal
@@ -17,6 +20,10 @@ class TrackingMeta(type):
         mcs._validation_list = []
         for name, value in dict.items():
             if isinstance(value, Node):
+                if name in TrackingMeta.reserved_attr_names:
+                    raise AttributeError(
+                        '{0} is a forbidden attribute name for a Node because'
+                        ' it overlaps with a Form attribute. Please rename.')
                 value._attr_name = name
                 t[value._create_counter] = value
                 if hasattr(value, 'validators'):
@@ -99,24 +106,43 @@ class Form(object):
 
     def __init__(self,
                  name=None,
-                 auto_start_close=True,
-                 start_template='form_open',
-                 close_template='form_close',
+                 auto_start_close=None,
+                 start_template=None,
+                 close_template=None,
                  g_context=None,
                  context=None,
                  start=None,
                  close=None,
                  **kwargs):
-        self.auto_start_close = auto_start_close
-        self.start_template = start_template
-        self.close_template = close_template
-        self.g_context = g_context if g_context else {}
-        self.context = context if context else {}
+
+        """ Basically, set the instance attribute to one of the following in
+        order of preference:
+        1. Passed in parameter
+        2. Class attribute
+        3. Set default """
+        def override(value, attr, default):
+            if value:
+                setattr(self, attr, value)
+            elif not hasattr(self, attr):
+                setattr(self, attr, default)
+
+        # override semantics
+        override(auto_start_close, 'auto_start_close', True)
+        override(start_template, 'start_template', 'form_open')
+        override(close_template, 'close_template', 'form_close')
+        override(g_context, 'g_context', {})
+        override(context, 'context', {})
 
         # set a default for our name to the class name
         self.name = name if name else self.__class__.__name__
+        self.context['name'] = self.name  # pass it to start/close
 
-        # passes everything to our rendering context and updates params
+        # pass some special keywords to our context if they're defined as class
+        # attributes
+        if hasattr(self, 'title'):
+            self.context['title'] = self.title
+        # passes everything to our rendering context and updates params.
+        # Overwrites class attributes
         self.context.update(kwargs)
 
         # since our default id is based off of the parent id
@@ -154,6 +180,17 @@ class Form(object):
                 raise AttributeError("close attribute is special and should "
                                      "specify a Node to begin your form")
 
+        # Add some useful global variables for templates
+        default_globals = {'form_id': self.name}
+        # Let our globals be overridden
+        default_globals.update(self.g_context)
+        self.g_context = default_globals
+
+        # Initialize some general state variable
+        self._last_valid = None
+        self._last_raw_json = None
+
+
     def render(self):
         """ Runs the renderer to parse templates of nodes and generate the form
         HTML.
@@ -163,6 +200,26 @@ class Form(object):
 
         return self._renderer().render(self._node_list, self.g_context)
 
+    def insert_validator(self, new_validators):
+        """ Inserts a validator to the validator list.
+
+        :param validator: The :class:`Check` to be inserted.
+        :type validator: Check """
+
+        # Check to make sure the passed in value is a list/tuple
+        if not isinstance(new_validators, tuple) and\
+           not isinstance(new_validators, list):
+            new_validators = [new_validators,]
+
+        for validator in new_validators:
+            # check to allow passing in just a check
+            if not isinstance(validator, Check):
+                raise TypeError
+
+            # append the validator to the list
+            self._validation_list.append(validator)
+
+
     def insert(self, position, new_node_list):
         """ Inserts a :class:`Node` object or a list of objects at the
         specified position into the :attr:`Form._node_list` of the form.
@@ -177,6 +234,17 @@ class Form(object):
             new_node_list = (new_node_list,)
 
         for i, new_node in enumerate(new_node_list):
+
+            # Check to prevent shooting yourself in the foot
+            if new_node._attr_name in TrackingMeta.reserved_attr_names:
+                raise AttributeError('{0} is a forbidden attribute name for a'
+                    'Node because it overlaps with a Form attribute. Please rename.')
+
+            # Another clarity error message
+            if not new_node._attr_name:
+                raise AttributeError('Dynamically added nodes should have an '
+                                     '_attr_name attribute.')
+
             if position == -1:
                 self._node_list.append(new_node)
             else:
@@ -222,68 +290,105 @@ class Form(object):
         try:
             attr = getattr(self, name)
         except AttributeError:
-            return None
+            raise AttributeError('Form attribute {0} couldn\'t be resolved to'
+                                 ' a Node'.format(name))
         if isinstance(attr, Node):
             return attr
-        return None
+        raise AttributeError('Form attribute {0} couldn\'t be resolved to'
+                                ' a Node'.format(name))
+
+    def success_header_generate(self):
+        """ Please see the documentation for :meth:`Form.error_header_generate`
+        as it covers this function as well as itself. """
+        pass
 
     def error_header_generate(self, errors, block):
-        """ This method is called when any validators on the form fail
-        to pass.  The method should generate a dictionary that will be
-        passed to your error renderer, whether that be javascript
-        callbacks or re-rendering the form with error info in the
-        rendering context.
+        """ This function, along with success_header_generate allow you to give
+        form wide information back to the user for both AJAX validated forms
+        and conventionally validated forms, although the mechanisms are
+        slightly different. Both functions are run at the end of a successful
+        or failed validation call in order to give more information for
+        rendering.
 
-        :param errors: This will be a list of all other Nodes that have errors.
-        :param block: Whether or not the form submission will be blocked.
-        :type block: boolean
+        For passing information to AJAX rendering, simply return a dictionary,
+        or any python object that can be serialized to JSON. This information
+        gets passed back to the JavaScript callbacks of yota_activate in
+        slightly different ways. success_header_generate's information will get
+        passed to the render_success callback, while error_header_generate will
+        get sent as an error to the render_error callback under the context
+        start.
+
+        For passing information into a regular rendering context simply access
+        the attribute manually similar to below.
 
-        By default this function does nothing, but a common method of
-        implementing it may be as follows:
         .. code-block:: python
 
             self.start.add_error(
                 {'message': 'Please resolve the errors below to continue.'})
 
-        This will provide a simple error message to your start Node.
+        This will provide a simple error message to your start Node. In
+        practice these functions could also be used to trigger events and other
+        interesting things, although that was not their intended function.
+
+        :param errors: This will be a list of all other Nodes that have errors.
+        :param block: Whether or not the form submission will be blocked.
+        :type block: boolean
+
+        .. note: By default this function does nothing.
         """
         pass
 
+    def data_by_attr(self):
+        """ Returns a dictionary of currently stored :attr:`Node.data`
+        attributes keyed by :attr:`Node._attr_name`. Used for returning data
+        after its been processed by validators. """
+
+        ret = {}
+        for node in self._node_list:
+            ret[node._attr_name] = node.data
+        return ret
+
+    def data_by_name(self):
+        """ Returns a dictionary of currently stored :attr:`Node.data`
+        attributes keyed by :attr:`Node.name`. Used for returning data
+        after its been processed by validators. """
+
+        ret = {}
+        for node in self._node_list:
+            ret[node.name] = node.data
+        return ret
+
     def _gen_validate(self, data, piecewise=False):
         """ This is an internal utility function that does the grunt work of
         running validation logic for a :class:`Form`. It is called by the other
         primary validation methods. """
 
+        # Allows user to set a modular processor on incoming data
+        data = self._processor().filter_post(data)
+
         # reset all error lists and data
         for node in self._node_list:
             node.errors = []
             node.data = ''
+            node.resolve_data(data)
 
-        # Allows user to set a modular processor on incoming data
-        data = self._processor().filter_post(data)
+        # try to load our visited list of it's piecewise validation
+        if '_visited_names' not in data and piecewise:
+            raise AttributeError("No _visited_names present in data submission"
+                                 ". Data is required for piecewise validation")
+        elif piecewise:
+            visited = json.loads(data['_visited_names'])
 
         # assume to be not blocking
         block = False
         # loop over our checks and run our validators
         for check in self._validation_list:
-            # try to iterate over their validators
-            try:
-                check.resolve_attr_names(data, self)
-            except DataAccessException as e:
-                # ignore the exception if we're in piecewise mode
-                if not piecewise:
-                    raise e
-                else:
-                    # make sure no success until all validators can run
-                    block = True
-                    continue
-            try:
-                # Run our validator
+            check.resolve_attr_names(data, self)
+            if piecewise is False or check.node_visited(visited):
                 check.validate()
-            except TypeError as e:
-                raise NotCallableException(
-                    "Validators provided must be callable, type '{0}'" +
-                    "instead. Caused by {1}".format(type(check.validator), e))
+            else:
+                # If even a single check can't be run, we need to block
+                block = True
 
         # a list to hold Nodes that actually have errors
         error_node_list = []
@@ -292,22 +397,13 @@ class Form(object):
             # default
             if node.errors:
                 error_node_list.append(node)
-            else:
-                # if the data field is blank, try and populate it
-                if node.data == '':
-                    try:
-                        node.resolve_data(data)
-                    except DataAccessException:
-                        pass
-                continue
 
-            if not block:  # no sense in checking if already blocking
-                for error in node.errors:
-                    block |= error.get('block', True)
+            for error in node.errors:
+                block |= error.get('block', True)
 
         return block, error_node_list
 
-    def json_validate(self, data, piecewise=False):
+    def json_validate(self, data, piecewise=False, raw=False):
         """ The same as :meth:`Form.validate_render` except the errors
         are loaded into a JSON string to be passed back as a query
         result. This output is designed to be used by the Yota
@@ -318,46 +414,81 @@ class Form(object):
             is designed to be used for the AJAX piecewise validation
             function, although it does not have to be.
         :type piecewise: boolean
-        """
+
+        :param raw: If set to True then the second return parameter will be a
+        Python dictionary instead of a JSON string
+
+        :return: A boolean whether or not the form submission is valid and the
+        json string (or raw dictionary) to pass back to the javascript side.
+        The boolean is an anding of submission (whether the submit button was
+        actually pressed) and the block parameter (whether or not any blocking
+        validators passed) """
 
         # Allows user to set a modular processor on incoming data
         data = self._processor().filter_post(data)
-        # pass our data into the global rendering context for filling in info
-        self.g_context['data'] = data
 
         errors = {}
-        block, invalid = self._gen_validate(data, piecewise=piecewise)
-        # auto-disable if this is not the submit action and it's a piecewise to
-        # prevent auto-submission
+        """ We want to automatically block the form from actually submitting
+        if this is piecewise validation. In addition if they are actually
+        submitting then we want to run it as non-piecewise validation """
         if data.get('submit_action', 'false') != 'true' and piecewise:
+            block, invalid = self._gen_validate(data, piecewise=piecewise)
             block = True
-        # loop over our nodes
+        else:
+            block, invalid = self._gen_validate(data, piecewise=False)
+
+        # loop over our nodes and insert information for the JS callbacks
         for node in invalid:
-            errors[node.id] = node.errors
+            errors[node._attr_name] = {'identifiers': node.json_identifiers(),
+                                       'errors': node.errors}
 
         # if needed we should run our all form message generator and return
         # json encoded error message
-        retval = {'success': not block}
+        retval = {'block': block}
         if len(errors) > 0:
-            errors['start'] = self.error_header_generate(errors, block)
+            header_err = self.error_header_generate(errors, block)
+            if header_err:
+                errors['start'] = {'identifiers': self.start.json_identifiers(),
+                                   'errors': header_err}
+
+        if not block:
+            blob = self.success_header_generate()
+            if blob:
+                retval['success_blob'] = blob
+                if hasattr(self, 'start'):
+                    retval['success_ids'] = self.start.json_identifiers()
+
         retval['errors'] = errors
-        return json.dumps(retval)
+
+        # Throw back a variable in the json if there is both a submit
+        # and no blocking errors. The main purpose here is the allow
+        # easy catching of success in the view code.
+        if data.get('submit_action', 'false') == 'true' and not block:
+            valid = True
+        else:
+            valid = False
+
+        # Hold our return dictionary in memeory for easy editing later
+        self._last_raw_json = retval
+
+        # Return our raw dictionary if requested, otherwise serialize for
+        # convenience...
+        if raw:
+            return valid, retval
+        else:
+            return valid, json.dumps(retval)
 
     def validate(self, data):
         """ Runs all the validators associated with the :class:`Form`.
 
-        :returns: A list of nodes that have errors on failure or True on
-            success
-        """
+        :return: Whether the validators are blocking submission and a list of
+        nodes that have validation messages """
 
         # Allows user to set a modular processor on incoming data
         data = self._processor().filter_post(data)
         block, invalid = self._gen_validate(data)
 
-        if not block:
-            return True
-        else:
-            return invalid
+        return (not block), invalid
 
     def validate_render(self, data):
         """ Runs all the validators on the `data` that is passed in and returns
@@ -372,7 +503,9 @@ class Form(object):
             where the key is the 'name' of the form field and the data is a
             string then no post-processing is neccessary.
         :type data: dictionary
-        """
+
+        :return: Whether the validators are blocking submission and a re-render
+        of the form with the validation data passed in """
 
         # Allows user to set a modular processor on incoming data
         data = self._processor().filter_post(data)
@@ -381,9 +514,60 @@ class Form(object):
 
         self.g_context['block'] = block
 
+        # update our state var for later update_success calls
+        self._last_valid = 'render'
+
         # run our form validators at the end
-        if len(invalid) > 0:
-            self.error_header_generate(invalid, block)
-            return self.render()
+        if not block:
+            self.success_header_generate()
         else:
-            return True
+            self.error_header_generate(invalid, block)
+        return (not block), self.render()
+
+    def update_success(self, update_dict, raw=False):
+        """ This method serves as an easy way to update your success attributes
+        that are passed to the start Node rendering context, or passed back in
+        JSON. It automatically recalls whether the last validation call was to
+        json_validate or validate_render and modifys the correct dictionary
+        accordingly.
+
+        :param update_dict: The dictionary of values to update/add.
+        :type data: dictionary
+
+        :param raw: Whether you would like a pre-compiled JSON
+            string returned, or the raw dictionary.
+        :type raw: bool
+
+        :return: Return value is either the new JSON string (or raw dict if
+        requested) if json_validate was your last validation call, or a
+        re-render of the form with updated error messages if validate_render
+        was your last call.  """
+
+        if self._last_valid == 'render':
+            try:
+                self.start.errors[-1].update(update_dict)
+            except KeyError:
+                raise KeyError("Error updating your error dictionary for the "
+                               "start Node. There were no errors to modify.")
+            except AttributeError:
+                raise AttributeError("This method is designed to update an "
+                                     "error dictionary, yet your errors are "
+                                     "not dictionaries")
+
+            return self.render()
+
+        # We're going to default to json render
+        else:
+            # Modify our last json dict
+            try:
+                self._last_raw_json['success_blob'].update(update_dict)
+            except KeyError:
+                raise KeyError("Either your json_validate method has not been "
+                               "run yet, or your success_header_generate does"
+                               " not produce output")
+
+            # Continue the raw semantic...
+            if raw:
+                return self._last_raw_json
+            else:
+                return json.dumps(self._last_raw_json)
