@@ -50,36 +50,36 @@ class Node(object):
     """
 
     _create_counter = 0
+    piecewise_trigger = 'blur'
     """ Allows tracking the order of Node creation """
 
     def __init__(self,
                  template=None,
                  validator=None,
-                 label=True,
+                 label=None,
                  _requires=None,
                  _ignores=None,
                  _attr_name=None,
                  **kwargs):
 
-        # Allow _ignores and _requires to be overwritten at the instance level,
-        # and also have a default
-        if _ignores:
-            self._ignores = _ignores
-        elif not hasattr(self, '_ignores'):
-            self._ignores = ['template', 'validator']
+        """ Basically, set the instance attribute to one of the following in
+        order of preference:
+        1. Passed in parameter
+        2. Class attribute
+        3. Set default """
+        def override(value, attr, default):
+            if value:
+                setattr(self, attr, value)
+            elif not hasattr(self, attr):
+                setattr(self, attr, default)
 
-        if _requires:
-            self._requires = _requires
-        elif not hasattr(self, '_requires'):
-            self._requires = []
+        override(_ignores, '_ignores', ['template', 'validator'])
+        override(_requires, '_requires', [])
+        override(template, 'template', None)
+        override(validator, 'validator', None)
+        override(label, 'label', True)
 
-        if not hasattr(self, 'validator'):
-            self.validator = validator
-
-        if template:
-            self.template = template
         self._attr_name = _attr_name
-        self.label = label
 
         # Allows the parent form to keep track of attribute order
         self._create_counter = Node._create_counter
@@ -100,6 +100,16 @@ class Node(object):
         :doc:`Validators` section. """
         self.errors.append(error)
 
+    def json_identifiers(self):
+        """ Allows passing arbitrary identification information to your JSON
+        error rendering callback. For instance, a common use case is the display
+        an error message in a pre-defined div with a specific id. Well you may
+        perhaps pass in an 'error_div_id' attribute to the JSON callback to use
+        when trying to render this error. The default for Yota builtin nodes is
+        to pass 'error_id' indicating the id of the error container in addition
+        to a list containing all input elements in the Node's ids. """
+        return {'error_id': self.id + '_error', 'elements': [self.id, ]}
+
     def set_identifiers(self, parent_name):
         """ This function gets called by the parent `Form` when it is
         initialized or inserted. It is designed to set various unique
@@ -124,6 +134,7 @@ class Node(object):
         if not hasattr(self, 'title'):
             self.title = self._attr_name.capitalize().replace('_', ' ')
 
+
     def resolve_data(self, data):
         """ This method is called when resolving the data from a form
         submission and linking it to a specific Node. The return value of this
@@ -133,7 +144,8 @@ class Node(object):
         try:
             self.data = data[self.name]
         except KeyError:
-            raise DataAccessException
+            raise DataAccessException("Node {0} cannot find name {1} in "
+                                      "submission data.".format(self._attr_name, self.name))
 
     def get_context(self, g_context):
         """ Builds our rendering context for the Node at render time. By
@@ -157,17 +169,34 @@ class Node(object):
         d['g'] = g_context
         return d
 
+    def get_list_names(self):
+        """ As the title suggests this needs to return an iterable of names. These
+        should be names corresponding to form elements that the Node will
+        generate. This list is uesed by piecewise validation to determine if a
+        Node has been visisted base on a list of names that have been visited,
+        attempting to bridge the concepts of Nodes and Elements. """
+        return (self.name, )
+
 
 class BaseNode(Node):
-    """ This base Node supplies the name of the base rendering template the
-    is used for standard form elements. This base template provides error
-    divs and the horizontal form layout for Bootstrap.
-    """
+    """ This base Node supplies the name of the base rendering template that
+    is used for standard form elements. This base template provides error divs
+    and the horizontal form layout for Bootstrap by default through the
+    `horiz.html` base template. """
     base = "horiz.html"
     css_class = ''
     css_style = ''
 
 
+class NonDataNode(Node):
+    """ A base to inherit from for Nodes that aren't designed to generate
+    output, such as the SubmitNode or the LeaderNode. It must override
+    resolve_data, otherwise a DataAccessException will be raised upon
+    validation time. """
+    def resolve_data(self, data):
+        pass
+
+
 class ListNode(BaseNode):
     """ Node for providing a basic drop down list. Requires an attribute that
      is a list of tuples providing the key and value for the dropdown list
@@ -188,34 +217,118 @@ class RadioNode(BaseNode):
         value of the second is the label.
     """
     template = 'radio_group'
-    type = 'radio'
     _requires = ['buttons']
 
+class CheckNode(BaseNode):
+    """ Creates a simple checkbox for your form. """
+    template = 'checkbox'
+
+    def resolve_data(self, data):
+        if self.name in data:
+            self.data = data[self.name]
+        else:
+            # Unchecked checkboxes don't submit any data so we'll set the
+            # value to false if there is no post data
+            self.data = False
+
+class CheckGroupNode(BaseNode):
+    """ Node for providing a group of checkboxes. Requires boxes
+    attribute. Instead of defining an ID value explicitly the
+    :class:`Node.set_identifiers` defines a prefix value to be prefixed to all
+    id elements for checkboxes in the group. The output data is a list
+    containing the names of the checkboxes that were checked.
+
+    :attr boxes: Must be a list of tuples where the first element is the
+        name, the second is the label.
+    """
+    template = 'checkbox_group'
+    _requires = ['boxes']
+
+    def resolve_data(self, data):
+        # return a list of checked values since we have multiple names
+        ret = []
+        for name, desc in self.boxes:
+            try:
+                if len(data[name]) > 0:
+                    ret.append(name)
+            except KeyError:
+                pass
+
+        self.data = ret
+
+    def json_identifiers(self):
+        ids = []
+        for name, desc in self.boxes:
+            ids.append(self.prefix + name)
+        return {'error_id': self.id + "_error", 'elements': ids}
+
+    def set_identifiers(self, parent_name):
+        # defines a prefix to be used on all the different checkbox ids
+        if not hasattr(self, 'prefix'):
+            self.prefix = parent_name + "_"
+        # defines a generic id to be used for generating things like error ids
+        if not hasattr(self, 'id'):
+            self.id = parent_name + "_" + self._attr_name
+        if not hasattr(self, 'title'):
+            self.title = self._attr_name.capitalize().replace('_', ' ')
+
 
 class ButtonNode(BaseNode):
-    """ Creates a simple button in your form.
+    """ Creates a button in your form that submits
+    no data.
     """
     template = 'button'
+    button_title = 'Click me!'
+
+    def resolve_data(self, data):
+        pass
 
 
 class EntryNode(BaseNode):
+    """ Creates an input box for your form. """
     template = 'entry'
 
 
+class PasswordNode(BaseNode):
+    """ Creates an input box for your form. """
+    template = 'password'
+
+
 class TextareaNode(BaseNode):
-    """ A node with a basic textarea template with defaults provided  """
+    """ A node with a basic textarea template with defaults provided.
+
+    :attr rows: The number of rows to make the textarea
+    :attr columns: The number of columns to make the textarea
+    """
     template = 'textarea'
     rows = '5'
     columns = '10'
 
+class CaptchaNode(BaseNode):
+    """ A node designed for basic captcha support. It expects
+    to receive a captcha id through the global context and uses
+    the id to reference a captcha image.
+    """
+    template = 'captcha'
+    placeholder = 'Are you human?'
 
-class SubmitNode(BaseNode):
+    def resolve_data(self, data):
+        try:
+            captcha = data['__captcha_id__']
+            captcha_attempt = data[self.name]
+        except KeyError:
+            raise DataAccessException("Node {0} cannot find name {1} in "
+                                      "submission data.".format(self._attr_name, self.name))
+        self.data = {'captcha': captcha, 'captcha_attempt':captcha_attempt}
+
+
+class SubmitNode(NonDataNode, BaseNode):
     template = 'submit'
 
 
-class LeaderNode(Node):
-    """ A Node that simply removes the title attribute from the Node rendering
-    context. Intended for use in the start and end Nodes. """
+class LeaderNode(NonDataNode):
+    """ A Node that does few special things to setup and close the form.
+    Intended for use in the start and end Nodes. """
 
     form_class = "form-horizontal"
     action = ''
